package alxtree;

import java.util.Vector;

/**
 * Titre : AlxNoeud
 * Description : Objet reprsentant un noeud d'une arboressence (pour un AlxTree)
 * @author <A href=alapetite@fr.st>Alexandre Alapetite</A>
 * @version 1.0
 * @see AlxTree
 */
public class AlxNoeud extends Object
{
  //attributs
  private String nom;
  private boolean ouvert;
  private Vector fils=new Vector(1,2);
  private Vector listenerList=new Vector(1,1); //de AlxNoeudListener
  private int contiens;
  private AlxNoeud pere;
  //variables
  private int indice;

  public AlxNoeud()
  {
    this("x");
  }

  public AlxNoeud(String nom)
  {
    this(nom,null);
  }

  public AlxNoeud(String nom, AlxNoeud pere)
  {
    super();
    this.nom=nom;
    this.pere=pere;
    this.indice=0;
    this.ouvert=false;
    this.contiens=0;
  }

  public void addChangeListener(AlxNoeudListener l)
  {
    listenerList.addElement(l);
  }

  protected void fireTreeChanged(AlxNoeudEvent ane)
  {
    for (int i=listenerList.size()-1; i>=0; i--)
    {
      ((AlxNoeudListener)listenerList.elementAt(i)).alxNoeudChanged(ane);
    }
    if (pere!=null) pere.fireTreeChanged(ane);
  }

  private void fireTreeChanged(int code)
  {
    fireTreeChanged(new AlxNoeudEvent(this,code));
  }

  public String getNom()
  {
    return this.nom;
  }

  public String getChemin()
  {
    if (pere!=null) return pere.getChemin()+"/"+this.nom;
    else return this.nom;
  }

  public AlxNoeud getPere()
  {
    return this.pere;
  }

  public void setPere(AlxNoeud alxNoeud)
  {
    this.pere=alxNoeud;
  }

  public boolean isOuvert()
  {
    return this.ouvert;
  }

  public void setOuvert(boolean ouvert)
  {
    this.ouvert=ouvert;
    if (ouvert)
    {
      if (pere!=null) pere.incContiensAffiche(fils.size()+contiens);
      fireTreeChanged(AlxNoeudEvent.ALX_NOEUD_OUVERTURE);
    }
    else
    {
      if (pere!=null) pere.decContiensAffiche(fils.size()+contiens);
      fireTreeChanged(AlxNoeudEvent.ALX_NOEUD_FERMETURE);
    }
  }

  public boolean isFeuille()
  {
    return (fils.size()<=0);
  }

  public int size()
  {
    return fils.size();
  }

  protected void incContiensAffiche(int nb)
  {
    this.contiens+=nb;
    if (pere!=null) pere.incContiensAffiche(nb);
  }

  public void decContiensAffiche(int nb)
  {
    this.contiens-=nb;
    if (pere!=null) pere.decContiensAffiche(nb);
  }

  public int getContiensAffiche()
  {
    if (ouvert) return contiens+size();
    else return 0;
  }

  public void addFils(String nom)
  {
    addFils(new AlxNoeud(nom,this));
  }

  public void addFils(AlxNoeud alxNoeud)
  {
    alxNoeud.setPere(this); //ncessaire ?  dbattre
    fils.addElement(alxNoeud); //et pas .add() car on utilise le JDK1.1
    fireTreeChanged(AlxNoeudEvent.ALX_NOEUD_AJOUT);
  }

  public void addFils(int indice, String nom)
  {
    addFils(indice,new AlxNoeud(nom,this));
  }

  public void addFils(int indice, AlxNoeud alxNoeud)
  {
    //alxNoeud.setPere(this); //ncessaire ?  dbattre
    fils.insertElementAt(alxNoeud,indice); //et pas .add(,)
    fireTreeChanged(AlxNoeudEvent.ALX_NOEUD_AJOUT);
  }

  public void removeFilsAt(int indice)
  {
    fils.removeElementAt(indice);
    fireTreeChanged(AlxNoeudEvent.ALX_NOEUD_SUPPRESSION);
  }

  public boolean removeFils(AlxNoeud alxNoeud)
  {
    if (fils.removeElement(alxNoeud))
    {
      fireTreeChanged(AlxNoeudEvent.ALX_NOEUD_SUPPRESSION);
      return true;
    }
    else return false;
  }

  public boolean removeFils(String nom)
  {
    AlxNoeud alxNoeud;
    return (((alxNoeud=getFils(nom))!=null) && removeFils(alxNoeud));
  }

  public AlxNoeud getFilsAt(int indice)
  {
    this.indice=indice;
    if (indice>0 && indice<=size()) return (AlxNoeud)(fils.elementAt(indice-1));
    else return null;
  }

  public AlxNoeud getFils(String nom)
  {
    boolean trouve=false;
    resetFils();
    while (!trouve && hasMoreFils()) trouve=getNextFils().getNom().equalsIgnoreCase(nom);
    return trouve ? getCurrentFils() : null;
  }

  protected AlxNoeud _getSousFilsOuvert(int position)
  {
    resetFils();
    if ((position<=0)||(!hasMoreFils())) return this;
    else
    {
      boolean cherche=true;
      AlxNoeud alxNoeud;
      do
      {
        position--;
        alxNoeud=getNextFils();
        if (position>alxNoeud.getContiensAffiche()) position-=alxNoeud.getContiensAffiche();
        else cherche=false;
      } while ((cherche) && hasMoreFils());
      if (!cherche) return getCurrentFils()._getSousFilsOuvert(position);
      else return this._getSousFilsOuvert(getContiensAffiche());
    }
  }

  public AlxNoeud getSousFilsOuvert(int position)
  {
    return _getSousFilsOuvert(position-1);
  }

  public boolean hasMoreFils()
  {
    return indice<fils.size();
  }

  public AlxNoeud getNextFils()
  {
    indice++;
    return getFilsAt(indice);
  }

  public void resetFils()
  {
    this.indice=0;
  }

  public AlxNoeud getCurrentFils()
  {
    return getFilsAt(indice);
  }
}
