package alxtree;

import java.awt.*;
import java.util.Vector;
import java.awt.event.MouseListener;
import java.awt.event.MouseEvent;
//import java.awt.image.BufferedImage; //dommage, n'existe pas en JDK1.1

/**
 * Titre : AlxTree
 * Description : composant graphique AWT permettant de reprsenter une arboressence
 * @author <A href=alapetite@fr.st>Alexandre Alapetite</A>
 * @version 1.0
 * @see AlxTreeListener
 */
public class AlxTree extends Canvas implements MouseListener,AlxNoeudListener
{
  /** bricolage pour ne pas faire <CODE>initGraphic()</CODE> dans le constructeur, mais dans le premier paint() */
  private boolean running=false;
  private PointeurInteger hauteur = new PointeurInteger(1); //variable
  /** racine de l'arbre  afficher */
  private AlxNoeud racine;
  /** liste des <CODE>AlxTreeListener</CODE> qui se sont enregistrs
   * @see AlxTreeListener
   */
  private Vector listenerList=new Vector(1,1); //de AlxTreeListener
  /** tableau de morceaux d'images */
  private AlxBufferedImage[] images=new AlxBufferedImage[8];
  private final static int IMG_LIENS_LARGEUR=19;
  private final static int IMG_HAUTEUR=16;
  private final static int IMG_LARGEUR=31;
  private final static int IMG_LIENS=0;
  private final static int IMG_FERME_RACINE=1;
  private final static int IMG_OUVERT_RACINE=2;
  private final static int IMG_FERME_PLEIN=3;
  private final static int IMG_OUVERT_PLEIN=4;
  private final static int IMG_FERME_VIDE=5;
  private final static int IMG_OUVERT_VIDE=6;
  private final static int IMG_DOUBLE_BUFFER=7;
  /** pour remplacer la mthode getWidth() indisponible en JDK 1.1 */
  private int width=400;
  /** pour remplacer la mthode getHeight() indisponible en JDK 1.1 */
  private int height=100;

  public AlxTree()
  {
    this(new AlxNoeud("R"));
    this.racine.setOuvert(true);
  }

  public AlxTree(AlxNoeud racine)
  {
    super();
    this.racine=racine;
    //initGraphique(); //ne pas faire ici car Component.createImage(...) est en retard d'o exception
    addMouseListener(this);
    this.racine.addChangeListener(this);
    this.setBackground(Color.white);
  }

  /** Permet de s'inscrire pour recevoir un vnement lorsque cet <CODE>AlxTree</CODE> est modifi */
  public void addChangeListener(AlxTreeListener l)
  {
    listenerList.addElement(l);
  }

  /** transmet une notification de modification  tous les <CODE>AlxTreeListener</CODE> inscrits
   *  @see AlxTreeListener
   */
  private void fireTreeChanged(AlxNoeud alxNoeud,int code)
  {
    AlxTreeEvent alxTreeEvent = new AlxTreeEvent(this,alxNoeud,code);
    for (int i=listenerList.size()-1; i>=0; i--)
    {
      ((AlxTreeListener)listenerList.elementAt(i)).alxTreeChanged(alxTreeEvent);
    }
  }

  /** en rponse  une modification d'un noeud de cet <CODE>AlxTree</CODE>
   *  @param ane <CODE>AlxNoeud</CODE> qui a subit une modification
   */
  public void alxNoeudChanged(AlxNoeudEvent ane)
  {
    if (running)
    {
      affiche();
      fireTreeChanged(ane.getAlxNoeud(),ane.getID());
    }
  }

  /** en rponse  un vnement souris sur cet <CODE>AlxTree</CODE> */
  public void mouseClicked(MouseEvent e)
  {
    int y=e.getY();
    if ((y%IMG_HAUTEUR)>0) y=(y/IMG_HAUTEUR)+1; //A remplacer par un floor !!!
    else y=(y/IMG_HAUTEUR);

    AlxNoeud alxNoeud=racine.getSousFilsOuvert(y);
    alxNoeud.setOuvert(!alxNoeud.isOuvert());
  }

  public void mouseEntered(MouseEvent e){}
  public void mouseExited(MouseEvent e){}
  public void mousePressed(MouseEvent e){}
  public void mouseReleased(MouseEvent e){}

  /** retourne l'image du code donn du tableau <CODE>images</CODE>
   *  @see #images
   */
  private Image getImage(int code)
  {
    AlxBufferedImage alxBufferedImage = images[code];
    return alxBufferedImage.image;
  }
 //Mthodes graphiques

  /** permet de remplir avec la couleur de fond une <CODE>AlxBufferedImage</CODE>
   *  @param bufferedImage image  remplir
   *  @param graphics <CODE>Graphics</CODE> de l'image <CODE>bufferedImage</CODE>
   *  @see #getBackground()
   */
  private void toutBlanc(AlxBufferedImage bufferedImage,Graphics graphics)
  {
    graphics.setColor(this.getBackground());
    graphics.fillRect(0,0,bufferedImage.getWidth(),bufferedImage.getHeight());
  }

  private void dessineCarreMoins(AlxBufferedImage bufferedImage,Graphics graphics)
  {
    graphics.setColor(Color.white);
    graphics.fillRect(1,5,7,7); //dessine le carr
    graphics.setColor(new Color(8355711));
    graphics.drawRect(0,4,8,8); //dessine le carr
    for (int cpt=10;cpt<=14;cpt+=2) bufferedImage.setRGB(cpt,8,8355711); //et les pointillets
    graphics.setColor(Color.black);
    graphics.drawLine(2,8,6,8); //dessine le moins
  }

  private void dessineCarrePlus(AlxBufferedImage bufferedImage,Graphics graphics)
  {
    dessineCarreMoins(bufferedImage,graphics);
    dessinePlus(bufferedImage,graphics);
  }

  private void dessinePlus(AlxBufferedImage bufferedImage,Graphics graphics)
  {
    graphics.drawLine(4,6,4,10); //dessine le plus
  }

  private void dessineLiensVert(AlxBufferedImage bufferedImage)
  {
    bufferedImage.setRGB(4,0,8355711);
    bufferedImage.setRGB(4,2,8355711);
    bufferedImage.setRGB(4,14,8355711);
  }

  private void dessineLiensVertComplet(AlxBufferedImage bufferedImage)
  {
    dessineLiensVert(bufferedImage);
    for(int cpt=4;cpt<=12;cpt+=2) bufferedImage.setRGB(4,cpt,8355711);
  }

  private void dessineLiensHorizComplet(AlxBufferedImage bufferedImage)
  {
    dessineLiensVert(bufferedImage);
    for(int cpt=6;cpt<=14;cpt+=2) bufferedImage.setRGB(cpt,8,8355711);
  }

  private void dessineDossierOuvert(AlxBufferedImage bufferedImage, Graphics graphics)
  {
    graphics.setColor(Color.black);
    graphics.fillRect(18,4,13,10); //dessine l'hombre
    graphics.setColor(new Color(16777116));
    graphics.fillRect(17,3,12,9); //dessine le fond du dossier
    //graphics.drawLine(17,7,17,8);
    graphics.setColor(Color.black); //noirs
    graphics.drawLine(27,8,27,7);
    graphics.drawLine(28,9,28,10);
    graphics.setColor(Color.white); //blancs
    graphics.drawLine(25,4,28,4);
    graphics.drawLine(18,2,22,2);
    graphics.drawLine(17,10,17,9);
    graphics.drawLine(16,8,16,7);
    graphics.drawLine(17,5,17,3);
    bufferedImage.setRGB(18,11,16777215);
    bufferedImage.setRGB(23,3,16777215);
    graphics.setColor(new Color(10263552)); //les lignes bords
    graphics.drawLine(17,12,28,12);
    graphics.drawLine(15,6,26,6);
    bufferedImage.setRGB(17,11,10263552);
    graphics.drawLine(18,1,22,1);
    graphics.drawLine(24,3,29,3);
    graphics.drawLine(15,8,15,7);
    graphics.drawLine(16,10,16,9);
    graphics.drawLine(10,16,9,16);
    graphics.drawLine(16,5,16,3);
    bufferedImage.setRGB(17,2,10263552);
    bufferedImage.setRGB(23,2,10263552);
    bufferedImage.setRGB(27,9,10263552);
    bufferedImage.setRGB(28,11,10263552);
    graphics.drawLine(29,9,29,10);
    graphics.setColor(new Color(13553251)); //les lignes vert clair
    graphics.drawLine(29,4,29,8);
    graphics.drawLine(28,7,28,8);
    graphics.drawLine(27,10,27,11);
    graphics.drawLine(29,4,29,8);
    graphics.drawLine(26,7,26,8);
    graphics.setColor(new Color(16764572)); //dgrad
    graphics.drawLine(19,11,23,7);
    graphics.drawLine(21,11,25,7);
    graphics.drawLine(23,11,25,9);
    graphics.drawLine(24,11,26,9);
    graphics.fillRect(25,10,2,2);
  }

  private void dessineDossierFerme(AlxBufferedImage bufferedImage, Graphics graphics)
  {
    bufferedImage.setRGB(17,2,16777215); //pixel blanc
    graphics.setColor(Color.black);
    graphics.drawLine(16,13,29,13); //dessine l'hombre en bas
    graphics.drawLine(29,4,29,12); //dessine l'hombre  droite
    graphics.setColor(new Color(13553251));
    graphics.drawLine(16,12,28,12); //dessine le bord en bas
    graphics.drawLine(28,4,28,11); //dessine le bord  droite
    graphics.drawLine(16,3,22,3); //dessine le bord en haut
    graphics.setColor(new Color(10263552));
    graphics.drawLine(15,3,15,12); //dessine le bord  gauche
    graphics.drawLine(17,1,21,1); //dessine l'onglet en haut
    bufferedImage.setRGB(16,2,10263552);
    bufferedImage.setRGB(22,2,10263552);
    graphics.drawLine(23,3,28,3);
    graphics.setColor(new Color(16777116));
    graphics.fillRect(16,4,12,8); //dessine le centre du dossier
    graphics.drawLine(20,2,21,2); //centre de l'onglet
    graphics.setColor(new Color(16777166));
    graphics.drawLine(16,11,16,4); //dessine le bord  gauche
    graphics.drawLine(17,4,26,4); //dessine le bord en haut
    graphics.drawLine(18,2,19,2); //centre de l'onglet
    graphics.setColor(new Color(16764572));
    graphics.drawLine(21,11,27,5); //diagonales de dgrad
    graphics.drawLine(23,11,27,7); //diagonales de dgrad
    graphics.drawLine(25,11,27,9); //diagonales de dgrad
    graphics.fillRect(26,10,2,2);
    bufferedImage.setRGB(22,1,0);
    bufferedImage.setRGB(23,2,0);
  }

  //mthodes graphiques compltes

  private void dessineImgLiens(AlxBufferedImage bufferedImage)
  {
    Graphics graphics=bufferedImage.getGraphics();
    toutBlanc(bufferedImage,graphics);
    graphics.setColor(new Color(8355711));
    for (int cpt=0;cpt<16;cpt+=2) bufferedImage.setRGB(4,cpt,8355711);
  }

  private void dessineImgFermeRacine(AlxBufferedImage bufferedImage)
  {
    Graphics graphics=bufferedImage.getGraphics();
    toutBlanc(bufferedImage,graphics);
    dessineCarrePlus(bufferedImage,graphics);
    dessineDossierFerme(bufferedImage,graphics);
  }

  private void dessineImgFermePlein(AlxBufferedImage bufferedImage)
  {
    dessineImgFermeRacine(bufferedImage);
    dessineLiensVert(bufferedImage);
  }

  private void dessineImgFermeVide(AlxBufferedImage bufferedImage)
  {
    Graphics graphics=bufferedImage.getGraphics();
    toutBlanc(bufferedImage,graphics);
    dessineLiensVertComplet(bufferedImage);
    dessineLiensHorizComplet(bufferedImage);
    dessineDossierFerme(bufferedImage,graphics);
  }

  private void dessineImgOuvertRacine(AlxBufferedImage bufferedImage)
  {
    Graphics graphics=bufferedImage.getGraphics();
    toutBlanc(bufferedImage,graphics);
    dessineCarreMoins(bufferedImage,graphics);
    dessineDossierOuvert(bufferedImage,graphics);
  }

  private void dessineImgOuvertPlein(AlxBufferedImage bufferedImage)
  {
    Graphics graphics=bufferedImage.getGraphics();
    dessineImgOuvertRacine(bufferedImage);
    dessineLiensVert(bufferedImage);
  }

  private void dessineImgOuvertVide(AlxBufferedImage bufferedImage)
  {
    Graphics graphics=bufferedImage.getGraphics();
    toutBlanc(bufferedImage,graphics);
    dessineDossierOuvert(bufferedImage,graphics);
    dessineLiensVertComplet(bufferedImage);
    dessineLiensHorizComplet(bufferedImage);
  }

  //fin code images
  /** code d'initialisation des composants graphiques */
  void initGraphique()
  {
    //this.setBackground(Color.white);
    this.setForeground(Color.black);
    this.setBounds(0,0,IMG_LARGEUR,IMG_HAUTEUR);
    images[IMG_LIENS] = new AlxBufferedImage(IMG_LIENS_LARGEUR,IMG_HAUTEUR,this);
    for (int cpt=1;cpt<7;cpt++) images[cpt] = new AlxBufferedImage(IMG_LARGEUR,IMG_HAUTEUR,this);
    images[IMG_DOUBLE_BUFFER] = new AlxBufferedImage(IMG_LARGEUR,IMG_HAUTEUR,this); //facultatif
    dessineImgLiens(images[IMG_LIENS]);
    dessineImgFermeRacine(images[IMG_FERME_RACINE]);
    dessineImgFermePlein(images[IMG_FERME_PLEIN]);
    dessineImgFermeVide(images[IMG_FERME_VIDE]);
    dessineImgOuvertRacine(images[IMG_OUVERT_RACINE]);
    dessineImgOuvertPlein(images[IMG_OUVERT_PLEIN]);
    dessineImgOuvertVide(images[IMG_OUVERT_VIDE]);
    affiche();
  }

  /** retourne la racine de l'arbre */
  public AlxNoeud getRacine()
  {
    return racine;
  }

  /** pointeur sur <CODE>int</CODE> */
  class PointeurInteger extends Object
  {
    /** l'entier <CODE>int</CODE> en question */
    public int valeur;

    public PointeurInteger()
    {
      this(0);
    }

    public PointeurInteger(int valeur)
    {
      super();
      this.valeur=valeur;
    }

    public void additionner(int terme)
    {
      this.valeur+=terme;
    }
  }

  /** mthode d'affichafe rcursive sur un <CODE>Graphics</CODE>
   *  @param graphics sur lequel on va dessiner
   *  @param alxNoeud noeud qui va tre dessin
   *  @param largeur dcallage du noeud par rapport  l'origine gauche
   *  @param hauteur dcallage du noeud par rapport  l'origine haut
   */
  private void affiche(Graphics graphics,AlxNoeud alxNoeud, int largeur, PointeurInteger hauteur)
  {
    int y=hauteur.valeur*IMG_HAUTEUR;
    int x;
    for (x=1;x<largeur;x++) graphics.drawImage(getImage(IMG_LIENS),x*IMG_LIENS_LARGEUR,y,this);
    x*=IMG_LIENS_LARGEUR;
    graphics.drawString(alxNoeud.getNom(),x+IMG_LARGEUR+6,y+IMG_HAUTEUR-3);
    hauteur.additionner(1);
    alxNoeud.resetFils();
    if(alxNoeud.isOuvert())
    {
      if(alxNoeud.hasMoreFils()) graphics.drawImage(getImage(IMG_OUVERT_PLEIN),x,y,this); //ce noeud a des enfants
      else graphics.drawImage(getImage(IMG_OUVERT_VIDE),x,y,this);
      largeur++;
      while(alxNoeud.hasMoreFils())
      {
        affiche(graphics,alxNoeud.getNextFils(),largeur,hauteur);
      }
    }
    else
    {
      if(alxNoeud.hasMoreFils()) graphics.drawImage(getImage(IMG_FERME_PLEIN),x,y,this); //ce noeud a des enfants
      else graphics.drawImage(getImage(IMG_FERME_VIDE),x,y,this);
    }
  }

  /** mthode d'affichage de dpart sur un buffer pour un arbre complet.<BR>
   *  Appelle affiche(Graphics graphics,AlxNoeud alxNoeud, int largeur, PointeurInteger hauteur)
   *  @see #affiche(Graphics graphics,AlxNoeud alxNoeud, int largeur, PointeurInteger hauteur)
   */
  private void affiche(Graphics graphics)
  {
    //graphics.setColor(this.getBackground()); //efface tout
    //graphics.fillRect(0,0,this.width,this.height);
    graphics.setColor(this.getForeground());
    graphics.drawString(racine.getNom(),IMG_LARGEUR+6,IMG_HAUTEUR-3);
    hauteur.valeur=1;
    racine.resetFils();
    if(racine.isOuvert())
    {
      graphics.drawImage(getImage(IMG_OUVERT_RACINE),0,0,this);
      while(racine.hasMoreFils())
      {
        affiche(graphics,racine.getNextFils(),1,hauteur);
      }
    }
    else
    {
      graphics.drawImage(getImage(IMG_FERME_RACINE),0,0,this);
    }
  }

  /** repeint le buffer et l'affiche */
  protected void affiche()
  {
    this.setSize(this.width,(racine.getContiensAffiche()+1)*IMG_HAUTEUR);
    images[IMG_DOUBLE_BUFFER] = new AlxBufferedImage(this.width,this.height,this);
    toutBlanc(images[IMG_DOUBLE_BUFFER],images[IMG_DOUBLE_BUFFER].getGraphics());
    affiche(images[IMG_DOUBLE_BUFFER].getGraphics());
    repaint();
  }

  /** rcriture car indisponible en JDK1.1 */
  public int getHeight()
  {
    return this.height;
  }

  /** rcriture car indisponible en JDK1.1 */
  public int getWidth()
  {
    return this.width;
  }

  public void setSize(int width, int height)
  {
    this.height=height;
    this.width=width;
    super.setSize(width,height);
  }

  /** affiche le composant en utilisant son buffer */
  public void paint(Graphics graphics)
  {
    super.paint(graphics);
    //Rectangle modif = graphics.getClipBounds();
    if (!running) //initialisation des composants lors du premier paint();
    {
      this.initGraphique(); //et oui, je sais, l'endroit n'est pas trs bien choisi
      running=true;
    }
    AlxBufferedImage doubleBuffer = images[IMG_DOUBLE_BUFFER];
    graphics.drawImage(doubleBuffer.image,0,0,doubleBuffer.getWidth(),doubleBuffer.getHeight(),this);
  }

}
